#!/bin/bash
#  retno.:
#   109: EXIT_CONF_MODIFIED
#   110: EXIT_CONF_SET_NEVER
#   111: EXIT_CONF_INCORRECT
#   112: EXIT_CONF_SKIPPED
#   113: EXIT_CONF_SET_E2
#   114: EXIT_CONF_SET_CONFLICT_KEY
#   others: not cared by dad yet.

OS=""
DISTRIBUTION=""
DISTRIBUTION_VERSION=""

AUDITD_CONF_PATH=/etc/audit/auditd.conf
AUDISPD_CONF_PATH=/etc/audisp/audispd.conf
AUDIT_RULES_PATH=/etc/audit/audit.rules
AUDIT_RULESD_PATH=/etc/audit/rules.d
AUDITD_PLUGIN_PATH=/etc/audisp/plugins.d

backup()
{
    FILE=$1

    if [ -e ${FILE}.pre_cda ]; then
        return
    fi
    grep CENTRIFYDA $FILE > /dev/null
    if [ $? == 0 ]; then
        return;
    fi
    cp $FILE ${FILE}.pre_cda
}

cleanup_tmp_files()
{
    rm -rf ${AUDIT_RULES_PATH}.running >/dev/null 2>&1
    rm -rf ${AUDIT_RULES_PATH}.failsafe >/dev/null 2>&1
    rm -rf /var/centrifyda/audit.rules.da.tmp >/dev/null 2>&1
}

revert_files()
{
    cleanup_tmp_files

    if [ "x$1" != "x1" ]; then
        # this file is special, it's always present when 'enabled'.
        rm -rf /var/centrifyda/audit.rules.da.tmp.del >/dev/null 2>&1

        [ "x$AUDITD_PLUGIN_PATH" != "x" ] && 
            rm -rf $AUDITD_PLUGIN_PATH/centrifyda.conf >/dev/null 2>&1
        [ "x$AUDIT_RULESD_PATH" != "x" ] && 
            rm -rf $AUDIT_RULESD_PATH/centrifyda.rules >/dev/null 2>&1

        mv -f ${AUDIT_RULES_PATH}.pre_cda $AUDIT_RULES_PATH >/dev/null 2>&1
        mv -f ${AUDITD_CONF_PATH}.pre_cda $AUDITD_CONF_PATH >/dev/null 2>&1
        mv -f ${AUDISPD_CONF_PATH}.pre_cda $AUDISPD_CONF_PATH >/dev/null 2>&1
    fi
}

check_compatiblity_linux()
{
    if [ "$DISTRIBUTION" = "ubuntu" -a "$DISTRIBUTION_VERSION" = "12.04" ]; then
        echo "ERROR: Ubuntu 12.04 (Precise) kernel is known to have compatiblity problem with auditd.";
        exit 900;
    fi
}

get_distribution_linux()
{
    if [ "$OS" != "Linux" ]; then
        return;
    fi

    if [ -f /etc/os-release ]; then
        . /etc/os-release
        DISTRIBUTION=$ID;
        DISTRIBUTION_VERSION=$VERSION_ID
        return;
    fi

    if [ -f /etc/redhat-release ]; then
        DISTRIBUTION="rhel";
        DISTRIBUTION_VERSION=`cat /etc/redhat-release | sed s/.*release\ // | sed s/\ .*//`
        return;
    fi

    if [ -f /etc/fedora-release ]; then
        DISTRIBUTION="fedora";
        DISTRIBUTION_VERSION=`cat /etc/fedora-release | sed s/.*release\ // | sed s/\ .*//`
        return;
    fi

    if [ -f /etc/debian_version ]; then
        DISTRIBUTION="debian";
        DISTRIBUTION_VERSION=`cat /etc/debian_version`
        return;
    fi

    if [ -f /etc/SuSE-release ]; then
        DISTRIBUTION="sles";
        DISTRIBUTION_VERSION=`cat /etc/SuSe-release | sed s/.*=// | awk '{print $1}'`
        return;
    fi

    DISTRIBUTION="generic"
    DISTRIBUTION_VERSION="1"

}

parse_e2()
{
    FILE=$1
    
    if [ ! -e ${FILE} ]; then
        return
    fi
    
    grep -e "^-e *2$" $FILE > /dev/null
    if [ $? == 0 ]; then
        return 113;
    fi
    
    return ;	
}

parse_conflict_key()
{
    FILE=$1
    
    if [ ! -e ${FILE} ]; then
        return
    fi
    
    # parse -a rules, find the cda monitored dir.
    local CHECK_DIR_A=`grep "cda_file_monitor_" $FILE | grep "^-a" | awk -F 'dir=' '{print $2}' | awk '{print $1}'`
    
    # parse -w rules, find the cda monitored dir.
    local CHECK_DIR_W=`grep "cda_file_monitor_" $FILE | grep "^-w" | awk '{print $2}'` 
    
    local CHECK_DIR="$CHECK_DIR_A$CHECK_DIR_W"
    
    for DIR in $CHECK_DIR; do
        # parse -a rules
        grep -v "cda_file_monitor_" $FILE | grep "$DIR" | grep "perm=[r,x]*[a,w]" > /dev/null
        if [ $? == 0 ]; then
            return 114
        fi
        
        # parse -w rules
        grep -v "cda_file_monitor_" $FILE | grep "$DIR" | grep -e "-p *[r,x]*[a,w]" > /dev/null
        if [ $? == 0 ]; then
            return 114
        fi     
    done
    
    return ;	
}

parse_conflict_key_for_rhel5_running_rules()
{
    FILE=$1
    
    if [ ! -e ${FILE} ]; then
        return
    fi
    
    # parse "dir=" rules, find the cda monitored dir.
    local CHECK_DIR_D=`grep "cda_file_monitor_" $FILE | grep "dir=" | awk -F 'dir=' '{print $2}' | awk '{print $1}'`
    
    # parse "watch=" rules, find the cda monitored dir.
    local CHECK_DIR_W=`grep "cda_file_monitor_" $FILE | grep "watch=" | awk -F 'watch=' '{print $2}' | awk '{print $1}'` 
    
    local CHECK_DIR="$CHECK_DIR_D$CHECK_DIR_W"
    
    for DIR in $CHECK_DIR; do
        grep -v "cda_file_monitor_" $FILE | grep "$DIR" | grep "perm=[r,x]*[a,w]" > /dev/null
        if [ $? == 0 ]; then
            return 114
        fi    
    done
    
    return ;	

}

setup_linux()
{
    USE_PLUGIN=0
    USE_RULESD=0

    if [ -z $DISTRIBUTION -o -z $DISTRIBUTION_VERSION ]; then
        echo "ERROR: Distribution is not supported";
    fi

    if [ ! -x /sbin/auditctl ]; then
        echo "ERROR: /sbin/auditctl is not found. Please check installation."
        exit 101;
    fi

    VERSION=`auditctl -v | cut -d " " -f 3`
    VER_MAJOR=`echo $VERSION | cut -d "." -f 1`
    VER_MINOR=`echo $VERSION | cut -d "." -f 2`
    VER_PATCH=`echo $VERSION | cut -d "." -f 3`
    if [ -z "$VER_PATCH" ]; then
        VER_PATCH=0
    fi

    if [ -z "$VER_MAJOR" -o -z "$VER_MINOR" ]; then
        echo "Unknown version of libaudit."
        exit 102;
    fi

    printf "libaudit: Version %d.%d.%d is detected.\n" $VER_MAJOR $VER_MINOR $VER_PATCH

    if [ $VER_MAJOR -lt 1 ] || \
        [ $VER_MAJOR = 1 -a $VER_MINOR -lt 2 ] || \
        [ $VER_MAJOR = 1 -a $VER_MINOR = 2 -a $VER_PATCH -lt 8 ]; then
        echo "Only libaudit with version >= 1.2.8 is supported.";
        exit 102;
    fi
    if [ $VER_MAJOR = 1 -a $VER_MINOR -lt 7 ]; then
        echo "- Use old configuration path."
        AUDITD_CONF_PATH=/etc/auditd.conf
        AUDIT_RULES_PATH=/etc/audit.rules
        AUDIT_RULESD_PATH=
        AUDITD_PLUGIN_PATH=
    else 
        echo "- Use plugin."
        USE_PLUGIN=1
        if [ $VER_MAJOR -ge 2 -a -d "$AUDIT_RULESD_PATH" ]; then
            echo "- Use rules.d"
            USE_RULESD=1   
        fi
        
        # Since audit-3.0, there are following changes:
        #   1. Merge auditd and audispd code
        #   2. Move all audispd config files under /etc/audit/
        #   3. Move audispd.conf settings into auditd.conf
        if [ $VER_MAJOR -ge 3 ]; then
            AUDISPD_CONF_PATH=""
            AUDITD_PLUGIN_PATH=`grep -v '^#' $AUDITD_CONF_PATH | grep 'plugin_dir' | awk -F '=' '{print $2}' | tr -d '[:blank:]'`
        fi
    fi
    FORCE=0
    if [ "$2" = force ]; then
        FORCE=1
    fi
    RELOAD=0
    if [ "x$1" = "xreload" ]; then 
        RELOAD=1
    fi

    RETVAL=0
    local E2_RESULT=""
    local CONFLICT_KEY_RESULT=""
    case $1 in 
        start | reload)
            if [ ! -f /var/centrifyda/audit.rules.da.tmp ]; then
                echo "/var/centrifyda/audit.rules.da.tmp not found, nothing to do."
                exit 106;
            fi

            service auditd status
            if [ $? != 0 ]; then
                service auditd start # in case not started yet
            fi

            # dadispatcher shall not be running now.
            # this is rare, but mostly, it shall still work well, so we just warn out.
            if [ $RELOAD != 1 ]; then
                ps -e | grep -v grep | grep -w dadispatcher > /dev/null
                if [ $? = 0 ]; then
                    echo "dadispatcher plugin is already running, please verify advanced monitoring status using dainfo."
                fi
            fi

            # note the result on rh5 cannot be used to reimport the rules.
            # (contents are all renamed and diff from config)
            # so no simple way to do exact backup of running rules.
            auditctl -l > ${AUDIT_RULES_PATH}.running

            # in config, both accepted, in running, rh>6 uses 'never,task', rh5 uses 'task,never'.
            grep -e "^-a *never,task" -e "^LIST_RULES: *task,never" ${AUDIT_RULES_PATH}.running &> /dev/null
            if [ $? = 0 -a $FORCE = 0 ]; then
                echo "audit rules has been set disabled for system events by '-a task,never'.  Backing off and will not $1 auditing."
                revert_files $RELOAD
                exit 110
            fi
            
            # we have to check also the file, or else may be troubled by restart
            if [ -d $AUDIT_RULESD_PATH ]; then
                backup ${AUDIT_RULES_PATH}
                /sbin/augenrules >/dev/null 2>&1
            else
                # on rh5 just use conf file as 'running backup'.
                cp ${AUDIT_RULES_PATH} ${AUDIT_RULES_PATH}.running
                backup ${AUDIT_RULES_PATH}
            fi

            grep -e "^-a *never,task" -e "^-a *task,never" ${AUDIT_RULES_PATH} &> /dev/null
            if [ $? = 0 -a $FORCE = 0 ]; then
                echo "audit rules has been set disabled for system events by '-a task,never'.  Backing off and will not $1 auditing."
                revert_files $RELOAD
                exit 110
            fi

            [ $RELOAD != 1 ] && backup $AUDITD_CONF_PATH 
            grep "^disp_qos " $AUDITD_CONF_PATH &> /dev/null
            if [ $? = 0 ]; then
                sed -i -e 's/^disp_qos = .*/disp_qos = lossless/g' $AUDITD_CONF_PATH
            else
                echo "disp_qos = lossless" >> $AUDITD_CONF_PATH
            fi

            if [ $USE_PLUGIN = 0 ]; then
                grep "^dispatcher" $AUDITD_CONF_PATH &> /dev/null
                if [ $? = 0 ]; then
                    sed -i -e 's#^dispatcher = .*.#dispatcher = /usr/share/centrifydc/bin/dadispatcher#g' $AUDITD_CONF_PATH
                else
                    echo "dispatcher = /usr/share/centrifydc/bin/dadispatcher" >> $AUDITD_CONF_PATH
                fi
            else
                if [ -d $AUDITD_PLUGIN_PATH ]; then
                    cp /etc/centrifyda/libaudit_plugin.conf $AUDITD_PLUGIN_PATH/centrifyda.conf
                else
                    echo "$AUDITD_PLUGIN_PATH not found, nothing to do."
                    revert_files $RELOAD
                    exit 105;
                fi
            fi

            TMP_CONF_PATH=""
            if [ -z "$AUDISPD_CONF_PATH" ]; then
                TMP_CONF_PATH=$AUDITD_CONF_PATH
            elif [ -f $AUDISPD_CONF_PATH ]; then
                TMP_CONF_PATH=$AUDISPD_CONF_PATH
            fi
            
            if [ -n "$TMP_CONF_PATH" ]; then
                [ $RELOAD != 1 ] && backup $TMP_CONF_PATH
                grep "^q_depth " $TMP_CONF_PATH &> /dev/null
                if [ $? = 0 ]; then
                    sed -i -e 's/^q_depth = .*/q_depth = 8192/g' $TMP_CONF_PATH
                else
                    echo "q_depth = 8192" >> $TMP_CONF_PATH
                fi
            fi

            cp ${AUDIT_RULES_PATH}.running ${AUDIT_RULES_PATH}.failsafe
            sed -i -e 's/.*-a never,task.*//g' -e 's/.*-a never,task.*//g' ${AUDIT_RULES_PATH}.running
            if [ $FORCE = 1 ]; then
                auditctl -D
                chcon -t auditd_etc_t ${AUDIT_RULES_PATH}.running
                auditctl -R ${AUDIT_RULES_PATH}.running

                # eliminate the lurking trouble.
                sed -i -e 's/.*-a task,never.*//g' -e 's/.*-a never,task.*//g' ${AUDIT_RULES_PATH} ${AUDIT_RULESD_PATH}/*
            fi

            # on incorrect config, emit the err and revert to old.
            chcon -t auditd_etc_t /var/centrifyda/audit.rules.da.tmp
            RET=`auditctl -R /var/centrifyda/audit.rules.da.tmp`
            if [ $? != 0 ]; then
                echo "ERROR: Fail to apply CDA audit rules, the auditctl error is: \"$RET\""
                auditctl -D
                chcon -t auditd_etc_t ${AUDIT_RULES_PATH}.failsafe
                auditctl -R ${AUDIT_RULES_PATH}.failsafe # ok for 'No rules' line.

                revert_files $RELOAD
                exit 111
            fi

            # dump the rules into config for restart/reboot use.
            if [ -d $AUDIT_RULESD_PATH ]; then
                # rh6 will directly import from file in start.
                auditctl -l > ${AUDIT_RULES_PATH}
                
                # parse the running rules 
                parse_conflict_key ${AUDIT_RULES_PATH} &> /dev/null
                CONFLICT_KEY_RESULT=$?

                # rh7 will do augen in start, so have to be under rules.d
                # we try best to also save the non-cda ones,
                # but if the non-cda ones not under rules.d, it may be lost after restart,
                # this is expected and not related to cda, so we don't care
                cp /var/centrifyda/audit.rules.da.tmp $AUDIT_RULESD_PATH/centrifyda.rules
                chcon -t auditd_etc_t $AUDIT_RULESD_PATH/centrifyda.rules
                /sbin/augenrules >/dev/null 2>&1
            else
                cat /var/centrifyda/audit.rules.da.tmp >> $AUDIT_RULES_PATH
                auditctl -l > ${AUDIT_RULES_PATH}.temp
                # the auditctl -l result on RHEL5 is special.
                parse_conflict_key_for_rhel5_running_rules ${AUDIT_RULES_PATH}.temp
                CONFLICT_KEY_RESULT=$?
                rm ${AUDIT_RULES_PATH}.temp
            fi
            
            # parse the audit.rules file
            if [ $CONFLICT_KEY_RESULT = 0 ]; then
                parse_conflict_key ${AUDIT_RULES_PATH} &> /dev/null
                CONFLICT_KEY_RESULT=$?
            fi
            
            # reload will not affect the rules, start will.
            service auditd reload 
            if [ $? != 0 ]; then
                echo "ERROR: Fail to reload auditd service."
                revert_files $RELOAD
                exit 108
            fi

            cleanup_tmp_files

            ps -e | grep -v grep | grep -w dadispatcher > /dev/null
            if [ $? != 0 ]; then
                # libaudit is buggy, sometimes it has latency:
                sleep 1
            fi

            ps -e | grep -v grep | grep -w dadispatcher > /dev/null
            if [ $? != 0 ]; then
                # sometimes it will never auto start:
                # and now if we found childless audispd, then it probably is exiting as a result of last 'disable'.
                DISPD_PID=`pgrep -x audispd`
                if [ "x$DISPD_PID" != "x" ] && ps -eo ppid | grep -w $DISPD_PID > /dev/null; then
                    true
                else
                    # now we are the only plugin, shall be fine to restart auditd
                    echo "Now dadispatcher shall be sole plugin, restart the auditd service."
                    service auditd restart
                fi
            fi

            ps -e | grep -v grep | grep -w dadispatcher > /dev/null
            if [ $? != 0 ]; then
                # may have latency, don't panic here.
                echo "dadispatcher plugin is not running yet, please verify advanced monitoring status using dainfo."
                exit 0
            fi

            # at last parse audit.rules.pre_cda to check "-e 2" rules.
            parse_e2 ${AUDIT_RULES_PATH}.pre_cda          
            E2_RESULT=$?
            
            if [ $E2_RESULT = 0 ]; then
                RETVAL=$CONFLICT_KEY_RESULT
            else
                RETVAL=$E2_RESULT
            fi
            
            ;;
        stop)
            if [ ! -f /var/centrifyda/audit.rules.da.tmp.del ]; then
                echo "/var/centrifyda/audit.rules.da.tmp.del not found, nothing to do."
                exit 106;
            fi

            service auditd status
            # if not running, cleanup and exit.
            if [ $? != 0 ]; then
                exit 0
            fi

            chcon -t auditd_etc_t /var/centrifyda/audit.rules.da.tmp.del
            RET=`auditctl -R /var/centrifyda/audit.rules.da.tmp.del`
            if [ $? != 0 ]; then
                echo "ERROR: Fail to revert audit rules, the auditctl error is: \"$RET\""
                exit 111
            fi

            revert_files

            service auditd reload

            # tricky here: in quick stop then start (as from restart cda service),
            # some machine's (on fedora24, e.g) audispd will now be exiting if it found no active plugins left, 
            # but it will linger in that status even in later start, so the 'audited reload' there
            # is totally ignored, thus advanced auditing cannot be enabled then.
            # so we have to kill also the lingering childless audispd.
            DISPD_PID=`pgrep -x audispd`
            if [ "x$DISPD_PID" != "x" ]; then
                ps -eo ppid | grep -w $DISPD_PID > /dev/null
                if [ $? != 0 ]; then
                    echo "audispd is childless yet not exit, will kill it."
                    kill -9 $DISPD_PID
                fi
            fi

            sleep 1

            # at last parse audit.rules to check "-e 2" rules.
            parse_e2 ${AUDIT_RULES_PATH}
            
            RETVAL=$?;
            ;;
        status)
            service auditd status
            RETVAL=$?;
            if [ $RETVAL = 0 ]; then
                ps -e | grep -v grep | grep -w dadispatcher > /dev/null
                RETVAL=$?;
            fi
            ;;
        default)
            RETVAL=104;
            ;;
    esac

    exit $RETVAL
}

# Replace or insert a key-value pair in AIX's audit config file
set_config_aix()
{
    FILE=$1
    SECTION=$2
    PARAMETER=$3
    VALUE="$4"
    APPEND=$5
    ESCAPE_VALUE=`echo $VALUE | sed -e 's/\//\\\\\//g'`
    
    cp $FILE /var/centrifyda/tmp.$$

    awk '
    BEGIN {
        found=0;
    }
    { 
            while (1) 
            { 
                if (/'$SECTION':/) 
                { 
                    p=1; 
                    print; 
                    next; 
                    continue;  
                };  
                if (p && /[a-zA-Z]*:/) 
                { 
                    p=0;
                    if(found==0)
                    {
                        print "\t'$PARAMETER' = '$VALUE'";
                    }
                    print;
                    next; 
                }; 
                if (p && /'$PARAMETER'[ ]*=.*./) 
                { 
                    found=1;
                    if ('$APPEND'==1)
                    {
                        if (/'$PARAMETER'[ ]*=.*'$ESCAPE_VALUE'/)
                        {
                            print;
                        }
                        else
                            print $0,",'$VALUE'";
                    }
                else
                        print "\t'$PARAMETER' = '$VALUE'"; 
                } 
                else 
                    print; 
                next; 
            }
    }
    END {
        if (found==0)
            print "\t'$PARAMETER' = '$VALUE'";
    }' $FILE > /var/centrifyda/tmp.$$

    if [ $? != 0 ]; then
        exit 105;
    fi

    mv /var/centrifyda/tmp.$$ $FILE
}

start_aix()
{
    # Backup original config
    backup /etc/security/audit/config 

    echo "OLD:"
    cat /etc/security/audit/config

    # Stop AIX audit facility
    audit shutdown > /dev/null
    if [ $? != 0 ]; then
        return 201;
    fi

    # Backup

    # Set binmode to off, and streammode to on
    set_config_aix /etc/security/audit/config start binmode off 0
    set_config_aix /etc/security/audit/config start streammode on 0

    # Define centrifyda classes
    set_config_aix /etc/security/audit/config classes centrifyda_exec "PROC_Execute,PROC_LPExecute" 0

    # Define interested user and their classes based on /etc/centrifyda/libaudit.conf
    while read LINE;
    do
        USER=`echo $LINE | awk '{print $1}'`
        CLASS=`echo $LINE | awk '{print $3}'`
        set_config_aix /etc/security/audit/config users $USER $CLASS 1
    done < /var/centrifyda/audit.rules.da.tmp


    # Redirect audit event from /dev/audit to /var/centrifyda/eventpipe
    #rm -f /var/centrifyda/eventpipe /etc/centrifyda/dastreamcmds
    #mknod /var/centrifyda/eventpipe p
    #echo "cat /dev/audit > /var/centrifyda/eventpipe &" > /etc/centrifyda/dastreamcmds
    echo "" > /etc/centrifyda/dastreamcmds
    set_config_aix /etc/security/audit/config stream cmds /etc/centrifyda/dastreamcmds 0

    echo "NEW:"
    cat /etc/security/audit/config

    # Restart AIX audit facility
    audit start > /dev/null
    if [ $? != 0 ]; then
        return 202;
    fi

    return 0;
}

stop_aix()
{
    echo "OLD:"
    cat /etc/security/audit/config

    # Stop AIX audit facility
    audit shutdown > /dev/null
    if [ $? != 0 ]; then
        return 201;
    fi

    return 0;

}

setup_aix()
{
    if [ ! -x /usr/sbin/audit ]; then
        exit 101;
    fi
    if [ ! -f /var/centrifyda/audit.rules.da.tmp ]; then
        exit 102;
    fi

    RETVAL=0
    case $1 in 
        start)
            start_aix
            RETVAL=$?
            ;;
        stop)
            stop_aix
            RETVAL=$?
            ;;
        status)
            audit query
            RETVAL=$?
            ;;
        default)
            RETVAL=104
            ;;
    esac

    exit $RETVAL
}

if [ -z "$UID" ]; then
    UID=`id | sed -e 's/uid=//' -e 's/(.*$//'`
fi
if [ $UID != 0 ]; then
    echo "You must run auditevent.sh as root";
    exit;
fi

OS=`uname -s`

echo "OS: $OS"

case $OS in
#    AIX)
#        setup_aix $@
#        ;;
    Linux)
        get_distribution_linux
        echo "DISTRIBUTION: $DISTRIBUTION"
        echo "VERSION: $DISTRIBUTION_VERSION"

        check_compatiblity_linux
        setup_linux $@
        ;;
    *)
        echo "Non support platform";
        exit 105;
        ;;
esac

