#!/bin/sh /usr/share/centrifydc/perl/run

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;

use CentrifyDC::GP::Mapper;
use CentrifyDC::GP::GPIsolation qw(GetRegKey);

my $file;
my $action;
my $user;
$file = {
    'comment_markers' => [
      '#',
    ],
    'hierarchy_separator' => '.',
    'list_expr' => ', *| +',
    'list_separator' => ', ',
    'lock' => '/etc/centrifyda/centrifyda.conf.lock',
    'match_expr' => [
      '/^\s*([^\s:=]+)[:=]\s*(.*)/',
    ],
    'named_list_separator' => ',',
    'parent_expr' => '^(.*)\.([^\.]+)$',
    'path' => [
      '/etc/centrifyda/centrifyda.conf',
    ],
    'post_action' => [
      'DO_DARELOAD',
    ],
    'value_map' => {
      'MaxFormat' => {
        'default_data' => '1',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Agent',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'MaxFormat',
        'value_type' => 'named',
      },
      'SessionPingInterval' => {
        'default_data' => '300',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Agent',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'SessionPingInterval',
        'value_type' => 'named',
      },
      'agent.max.missed.update.tolerance' => {
        'default_data' => '4',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Agent',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'agent.max.missed.update.tolerance',
        'value_type' => 'named',
      },
      'agent.send.hostname' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Agent',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'agent.send.hostname',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'agent.video.capture' => {
        'default_data' => 'enabled',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Agent',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'agent.video.capture',
        'value_type' => 'named',
      },
      'autofix.nss.conf' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'autofix.nss.conf',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'cache.enable' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'cache.enable',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'cache.max.size' => {
        'default_data' => '80000',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'cache.max.size',
        'value_type' => 'named',
      },
      'cache.time.to.live' => {
        'default_data' => '600',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'cache.time.to.live',
        'value_type' => 'named',
      },
      'dad.client.idle.timeout' => {
        'default_data' => '1800',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.client.idle.timeout',
        'value_type' => 'named',
      },
      'dad.collector.connect.timeout' => {
        'default_data' => '60',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.collector.connect.timeout',
        'value_type' => 'named',
      },
      'dad.dumpcore' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.dumpcore',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.event.enabled' => {
        'default_data' => 'false',
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.event.enabled',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.gssapi.seal' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.gssapi.seal',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.gssapi.sign' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.gssapi.sign',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.process.fdlimit' => {
        'default_data' => '2048',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.process.fdlimit',
        'value_type' => 'named',
      },
      'dad.resource.cpulimit' => {
        'default_data' => '50',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.resource.cpulimit',
        'value_type' => 'named',
      },
      'dad.resource.cpulimit.tolerance' => {
        'default_data' => '5',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.resource.cpulimit.tolerance',
        'value_type' => 'named',
      },
      'dad.resource.fdlimit' => {
        'default_data' => '1024',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.resource.fdlimit',
        'value_type' => 'named',
      },
      'dad.resource.memlimit' => {
        'default_data' => '104857600',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.resource.memlimit',
        'value_type' => 'named',
      },
      'dad.resource.restart' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.resource.restart',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.resource.timer' => {
        'default_data' => '600',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.resource.timer',
        'value_type' => 'named',
      },
      'dad.session.audit.enabled' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/SessionAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dad.session.audit.enabled',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dad.timer.diskspace' => {
        'default_data' => '360',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.timer.diskspace',
        'value_type' => 'named',
      },
      'dad.timer.monitor.nss.conf' => {
        'default_data' => '60',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.timer.monitor.nss.conf',
        'value_type' => 'named',
      },
      'dad.timer.update.agent.status' => {
        'default_data' => '300',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dad.timer.update.agent.status',
        'value_type' => 'named',
      },
      'dash.allinvoked' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.allinvoked',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.auditstdin' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.auditstdin',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.auditstdin.except' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.auditstdin.except',
        'value_type' => 'named',
      },
      'dash.cmd.audit.blacklist' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.cmd.audit.blacklist',
        'value_type' => 'named',
      },
      'dash.cmd.audit.show.actual.user' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.cmd.audit.show.actual.user',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.cont.without.dad' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.cont.without.dad',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.force.audit' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.force.audit',
        'value_type' => 'named',
      },
      'dash.loginrecord' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.loginrecord',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.obfuscate.pattern' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.obfuscate.pattern',
        'value_type' => 'named',
      },
      'dash.obfuscate.regex' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.obfuscate.regex',
        'value_type' => 'named',
      },
      'dash.parent.skiplist' => {
        'default_data' => 'sapstartsrv gdm-binary gdm-session-wor kdm sdt_shell',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.parent.skiplist',
        'value_type' => 'named',
      },
      'dash.reconnect.dad.retry.count' => {
        'default_data' => '3',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dash.reconnect.dad.retry.count',
        'value_type' => 'named',
      },
      'dash.reconnect.dad.wait.time' => {
        'default_data' => '1',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'dash.reconnect.dad.wait.time',
        'value_type' => 'named',
      },
      'dash.shell.env.var.set' => {
        'default_data' => 'false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.shell.env.var.set',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'dash.ssh.command.skiplist' => {
        'default_data' => 'scp rsync sftp-server',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.ssh.command.skiplist',
        'value_type' => 'named',
      },
      'dash.user.alwaysallowed.list' => {
        'default_data' => 'root',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.user.alwaysallowed.list',
        'value_type' => 'named',
      },
      'dash.user.skiplist' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Shell',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'dash.user.skiplist',
        'value_type' => 'named',
      },
      'event.execution.monitor' => {
        'default_data' => 'false',
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.execution.monitor',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'event.execution.monitor.user.skiplist' => {
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.execution.monitor.user.skiplist',
        'value_type' => 'named',
      },
      'event.file.monitor' => {
        'default_data' => 'false',
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.file.monitor',
        'value_type' => 'named',
        'valueoff' => 'false',
        'valueon' => 'true',
      },
      'event.file.monitor.process.skiplist' => {
        'default_data' => '/usr/sbin/daspool',
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.file.monitor.process.skiplist',
        'value_type' => 'named',
      },
      'event.file.monitor.user.skiplist' => {
        'default_data' => 'root',
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.file.monitor.user.skiplist',
        'value_type' => 'named',
      },
      'event.monitor.commands' => {
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.monitor.commands',
        'value_type' => 'named',
      },
      'event.monitor.commands.user.skiplist' => {
        'post_command' => [
          'CentrifyDC::GP::General::RunCmd("/usr/sbin/dareload -m")',
        ],
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/AdvancedAudit',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'event.monitor.commands.user.skiplist',
        'value_type' => 'named',
      },
      'lang_setting' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'lang_setting',
        'value_type' => 'named',
      },
      'lrpc2.message.signing' => {
        'default_data' => 'disabled',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Common',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'lrpc2.message.signing',
        'value_type' => 'named',
      },
      'lrpc2.rebind.timeout' => {
        'default_data' => '300',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/LRPC2Client',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'lrpc2.rebind.timeout',
        'value_type' => 'named',
      },
      'lrpc2.timeout' => {
        'default_data' => '30',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/LRPC2Client',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'lrpc2.timeout',
        'value_type' => 'named',
      },
      'nss.alt.zone.auditlevel' => {
        'default_data' => 'use_sysrights',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.alt.zone.auditlevel',
        'value_type' => 'named',
      },
      'nss.nologin.shell' => {
        'default_data' => '/sbin/nologin /bin/false',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.nologin.shell',
        'value_type' => 'named',
      },
      'nss.program.ignore' => {
        'default_data' => 'useradd userdel adduser usermod mkuser rmuser chuser',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.program.ignore',
        'value_type' => 'named',
      },
      'nss.user.conflict.auditlevel' => {
        'default_data' => 'audit_if_possible',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.user.conflict.auditlevel',
        'value_type' => 'named',
      },
      'nss.user.override.auditlevel' => {
        'default_data' => '0',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.user.override.auditlevel',
        'value_type' => 'named',
      },
      'nss.user.override.userlist' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/NSS',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'nss.user.override.userlist',
        'value_type' => 'named',
      },
      'preferred.audit.store' => {
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/DirectAudit/Common',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'preferred.audit.store',
        'value_type' => 'named',
      },
      'spool.diskspace.logstate.reset.threshold' => {
        'default_data' => '2',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/SpoolDiskSpace',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'spool.diskspace.logstate.reset.threshold',
        'value_type' => 'named',
      },
      'spool.diskspace.min' => {
        'default_data' => '10',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/SpoolDiskSpace',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'spool.diskspace.min',
        'value_type' => 'named',
      },
      'spool.diskspace.softlimit' => {
        'default_data' => '12',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/SpoolDiskSpace',
        'reg_type' => [
          'REG_SZ',
        ],
        'reg_value' => 'spool.diskspace.softlimit',
        'value_type' => 'named',
      },
      'spool.maxdbsize' => {
        'default_data' => '0',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/SpoolDiskSpace',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'spool.maxdbsize',
        'value_type' => 'named',
      },
      'user.ignore.audit.level' => {
        'default_data' => '0',
        'reg_class' => 'machine',
        'reg_key' => 'Software/Policies/Centrify/CentrifyDA/Settings/UnixAgent/Daemon',
        'reg_type' => [
          'REG_DWORD',
        ],
        'reg_value' => 'user.ignore.audit.level',
        'value_type' => 'named',
      },
    },
    'write_data' => '$value: $data\n',
};

$action = $ARGV[0];
my $mode = $ARGV[2] ? $ARGV[2] : $ARGV[1];
$user = $ARGV[2] ? $ARGV[1] : undef;

if ($action eq "unmap")
{

    CentrifyDC::GP::Mapper::UnMap($file, $user);
}
else
{
    CentrifyDC::GP::Mapper::Map($file, $user);
}
